from DateTime import DateTime
# Zope
from OFS import SimpleItem
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.ZCatalog.CatalogPathAwareness import CatalogPathAware
# Silva
from Products.Silva.Content import Content
from Products.Silva import SilvaPermissions, SilvaObject
# OAICore
from Products.OAICore.core import ZDateTimeTodatetime
from Products.OAICore.query import SearchQuery
from Products.OAICore.item import Item as BaseItem
from Products.OAICore.helpers import keysSortedByValue

# Formulator
from Products.Formulator.Form import ZMIForm

# Copied and adapted from SilvaNews

import Globals
from AccessControl import ClassSecurityInfo
from Products.Silva.adapters import adapter

class XMLBuffer:
    """Small file-like object for XML output.
    
    Implicitly converts unicode to UTF-8 and replaces characters to 
    entities when required
    """

    def __init__(self):
        self._buffer = []

    def write(self, data):
        if type(data) != type(u''):
            data = unicode(str(data))
        self._buffer.append(data)
        
    def read(self):
        """The semantics are different from the plain file interface's read!
            
        This will return the full buffer always, and won't move the 
        pointer
        """
        ret = ''.join(self._buffer)
        ret = self._convert(ret)
        return ret

    def _convert(self, data):
        """Convert data to UTF-8.
        """
        data = data.encode('UTF-8')
        return data

def quote_xml ( data ):
    """Quote string for XML usage.
    """
    if not data:
        return data
    data = data.replace('&', '&amp;')
    data = data.replace('"', '&quot;')
    data = data.replace('<', '&lt;')
    data = data.replace('>', '&gt;')
    return data

# Item

class Item(BaseItem, Content, SimpleItem.SimpleItem):
    """An item.
    """

    security = ClassSecurityInfo()
    
    meta_type = 'Silva OAI Item'

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getObj')

InitializeClass(Item)
    
class Query(SearchQuery, CatalogPathAware, Content, SimpleItem.SimpleItem):
    """A query for items.

    A query consists of two parts -- one restricts the items by
    certain criteria, the other can be used to search in these items.
    """

    security = ClassSecurityInfo()
    default_catalog = 'service_catalog'
    meta_type = 'Silva OAI Query'

    #XXX Default value for existing query objects
    _display_list = False

    def __init__(self, id, title, service):
        Content.__init__(self, id, title)
        SearchQuery.__init__(self, service)
        self._intro_text = None
        self._display_list = False

    def manage_afterAdd(self, item, container):
        self._afterAdd_helper(item, container)
        self._set_creation_datetime()
        Query.inheritedAttribute('manage_afterAdd')(self, item, container)
        
    def manage_beforeDelete(self, item, container):
        self._beforeDelete_helper(item, container)
        Query.inheritedAttribute('manage_beforeDelete')(self, item, container)

    security.declarePrivate('createItem')
    def createItem(self, obj, fields):
        return Item(obj, fields)
        
    # MANIPULATORS
    
    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'setFilters')

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'setSet')

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'setMetadataFormat')

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'setIntroText')
    def setIntroText(self, text):
        self._intro_text = text

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'setDisplayList')
    def setDisplayList(self, display_list):
        self._display_list = display_list

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'updateSearchForm')
    def updateSearchForm(self, form):
        self.addFulltextSearchField(form)
    
    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'updateManageForm')
    def updateManageForm(self, form):
        self.addSetField(form)
        self.addFilterFields(form)
    
    # ACCESSORS
    
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getService')

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getCatalog')

    security.declarePublic(
        SilvaPermissions.AccessContentsInformation, 'getMetadataFormat')
        
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getSet')

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getFields')
    
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getFilters')

    security.declarePrivate('updateQuery')

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getSearchables')

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getInitialSortOn')

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getInitialSortOrder')
            
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getColumns')

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'displayList')
    def displayList(self):
        return self._display_list
        
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getIntroText')
    def getIntroText(self):
        return self._intro_text

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'searchResults')
    def searchResults(self):
        # XXX validation should move to outer UI layer?
        request = self.REQUEST
        advanced = request.get('advanced_search')
        form_input = self.getSearchForm(advanced=advanced).validate_all(request)
        search_query = self._getSearchQuery(form_input)
        fulltext = form_input.get('fulltext', None)
        if fulltext:
            search_query['fulltext'] = fulltext
        # Non-Formulator fields, that are in the request anyway    
        sort_on = request.form.get('sort_on')
        if sort_on:
            search_query['sort_on'] = sort_on
        sort_order = request.form.get('sort_order')
        if sort_order:
            search_query['sort_order'] = sort_order
        return self.queryResults(search_query)

    def _getSearchQuery(self, form_input):
        q = {}
        searchables = self.getSearchables()
        for field in self.getFields():
            id = field.getId()
            if id not in searchables or self._filters.has_key(id):
                continue
            unique_values = None
            if field.getIndexType() != 'KeywordIndex':
                continue
            unique_values = self.getUniqueValues(id)
            field.updatePublicQuery(q, form_input, unique_values)
        return q

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'is_deletable')
    def is_deletable(self):
        """A Query is always deletable. (Since it is always published, the
        choices for deletable are always or never, and never makes the least
        sense ;)
        """
        return 1

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'is_published')
    def is_published(self):
        """Always be published."""
        return 1

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'getManageForm')
    def getManageForm(self):
        query_form = ZMIForm('query_form', 'Query Form', 1)
        self.updateManageForm(query_form)
        return query_form.__of__(self)
    
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'getSearchForm')
    def getSearchForm(self, advanced=False):
        search_form = ZMIForm('search_form', 'Search Form', 1)
        if advanced:
            self.addSearchFields(search_form)
        self.updateSearchForm(search_form)
        return search_form.__of__(self)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'rss')
    def rss(self, REQUEST=None, sort_on=None, sort_order=None, results=10):
        """Return the contents of this query as an RSS/RDF (RSS 1.0) feed"""
        RDF_HEADER = (
            '<?xml version="1.0" encoding="UTF-8" ?>\n' 
            '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
            'xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns="http://purl.org/rss/1.0/">\n')
        schema = self.getSchema()
        if REQUEST is not None:
            REQUEST.RESPONSE.setHeader('Content-Type', 'text/xml;charset=UTF-8')
        # get the newest items
        if sort_on or sort_order:
            brains = self.queryResults(
                {'sort_on': schema.getRSSFieldId(sort_on),
                'sort_order': sort_order})[:results]
        else:
            brains = self.queryResults(
                {'sort_on': schema.getRSSFieldId('date'),
                'sort_order': 'descending'})[:results]
        # create RDF/XML for the channel
        xml = XMLBuffer()
        xml.write(RDF_HEADER)

        # get the metadata binding to get the metadata for this viewer
        mdbinding = self.service_metadata.getMetadata(self)
        creationdate = mdbinding.get('silva-extra', 'creationtime')
        # create RDF/XML frame
        xml.write('<channel rdf:about="%s">\n' % self.absolute_url())
        xml.write('<title>%s</title>\n' % quote_xml(self.get_title()))
        xml.write('<link>%s</link>\n' % self.absolute_url())
        xml.write('<description>%s</description>\n' %
            quote_xml(mdbinding.get('silva-extra', 'content_description')))
        xml.write('<dc:creator>%s</dc:creator>\n' %
            quote_xml(mdbinding.get('silva-extra', 'creator')))
        date = creationdate.HTML4()
        xml.write('<dc:date>%s</dc:date>\n' % quote_xml(date))

        # output <items> list
        # and store items in a list for later use
        itemlist = []
        xml.write('<items>\n<rdf:Seq>\n')
        for brain in brains:
            itemlist.append(brain)
            url = self.absolute_url() + brain.id
            xml.write('<rdf:li rdf:resource="%s" />\n' % url)
        xml.write('</rdf:Seq>\n</items>\n')
        xml.write('</channel>\n\n')
        # loop over the itemslist and create the RSS/RDF item elements
        for brain in itemlist:
            self._rss_item_helper(brain.getObject(), xml)
        # DONE return XML
        xml.write('</rdf:RDF>\n')
        return xml.read()

    def _rss_item_helper(self, item, xml):
        """convert a single Silva object to an RSS/RDF 'hasitem' element"""
        #XXX this should really go once we get some time
        schema = self.getSchema()
        xml.write(
            '<item rdf:about="%s/%s">\n' % (
            self.absolute_url(), item.id))
        # RSS elements
        for id in ['title', 'description']:
            data = getattr(item, schema.getRSSFieldId(id))
            if data:
                if type(data) == type([]):
                    for datum in data:
                        xml.write(
                            '<dc:%(id)s>%(data)s</dc:%(id)s>\n' %
                            {'id': id, 'data': quote_xml(datum)})
                elif type(data) == type(DateTime()):
                    xml.write(
                        '<dc:%(id)s>%(data)s</dc:%(id)s>\n' %
                        {'id': id, 'data': quote_xml(data.HTML4())})
                else:
                    xml.write(
                        '<dc:%(id)s>%(data)s</dc:%(id)s>\n' %
                        {'id': id, 'data': quote_xml(data)})
        xml.write(
            '<link>%s/%s</link>\n' % (self.absolute_url(), item.id))
        for id in ['subject', 'creator', 'date']:
            data = getattr(item, schema.getRSSFieldId(id))
            if data:
                if type(data) == type([]):
                    for datum in data:
                        xml.write(
                            '<dc:%(id)s>%(data)s</dc:%(id)s>\n' %
                            {'id': id, 'data': quote_xml(datum)})
                elif type(data) == type(DateTime()):
                    xml.write(
                        '<dc:%(id)s>%(data)s</dc:%(id)s>\n' %
                        {'id': id, 'data': quote_xml(data.HTML4())})
                else:
                    xml.write(
                        '<dc:%(id)s>%(data)s</dc:%(id)s>\n' %
                        {'id': id, 'data': quote_xml(data)})
        xml.write('</item>\n')

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'atom')
    def atom(self, REQUEST=None, sort_on=None, sort_order=None, results=10):
        """Return the contents of this query as an Atom feed"""

        ATOM_HEADER = (
            '<?xml version="1.0" encoding="utf-8"?>\n'
            '<feed xmlns="http://www.w3.org/2005/Atom">\n')
            
        schema = self.getSchema()
        if REQUEST is not None:
            REQUEST.RESPONSE.setHeader('Content-Type', 'text/xml;charset=UTF-8')
        # get the newest items
        if sort_on or sort_order:
            brains = self.queryResults(
                {'sort_on': prefix + sort_on,
                'sort_order': prefix + sort_order})[:results]
        else:
            brains = self.queryResults(
                {'sort_on': schema.getRSSFieldId('date'),
                'sort_order': 'descending'})[:results]
        xml = XMLBuffer()
        xml.write(ATOM_HEADER)

        # get the metadata binding to get the metadata for this viewer
        mdbinding = self.service_metadata.getMetadata(self)
        url = self.absolute_url()
        stripped_list = url.split('//')[1].split('/')
        domain = stripped_list[0]
        path = '/'.join(stripped_list[1:])
        creationdate = mdbinding.get('silva-extra', 'creationtime')
        ymd = creationdate.HTML4()
        id = domain + ',' + ymd + ':/' + path
        xml.write('<id>%s</id>\n' % quote_xml(url))
        xml.write('<title type="text">%s</title>\n' % quote_xml(self.get_title()))
        description = mdbinding.get('silva-extra', 'content_description')
        if description:
            xml.write('<subtitle>%s</subtitle>\n' % quote_xml(description))
        xml.write('<link rel="self" href="%s/atom" />\n' % url)
        xml.write('<author><name>%s</name></author>\n' %
            quote_xml(mdbinding.get('silva-extra', 'creator')))
        date = self.getService().getDatestamp().strftime(
            '%Y-%m-%dT%H:%M:%SZ%z')
        xml.write('<updated>%s</updated>\n' % quote_xml(date))
        for brain in brains:
            self._atom_item_helper(brain.getObject(), xml)
        xml.write('</feed>')
        return xml.read()

    def _atom_item_helper(self, item, xml):
        """convert a single Silva object to an Atom 'entry' element"""
        schema = self.getSchema()
        url = self.absolute_url()
        xml.write('<entry>\n')
        # Atom elements
        titles = getattr(item, schema.getRSSFieldId('title'))
        if type(titles) != type([]):
            titles = [titles]
        for title in titles:
            xml.write('<title>%s</title>\n' % quote_xml(title))
        xml.write(
            '<link rel="alternate" type="text/html" href="%s/%s" />\n' % (
            url, item.id))
        stripped_list = url.split('//')[1].split('/')
        domain = stripped_list[0]
        path = '/'.join(stripped_list[1:])
        published = getattr(item, schema.getRSSFieldId('date'))
        if type(published) == type([]):
            published = published[0]
        if type(published) == type(DateTime()):
            published = published.HTML4()
        xml.write('<id>tag:%s,%s:%s</id>' % (
            domain,
            quote_xml(published.split('T')[0]),
            path + '/' + item.id))
        xml.write('<published>%s</published>' % quote_xml(published))
        xml.write('<updated>%s</updated>' % item.datestamp().HTML4())
        names = getattr(item, schema.getRSSFieldId('creator'))
        if type(names) != type([]):
            names = [names]
        for name in names:
            if name:
                xml.write('<author><name>%s</name></author>' % quote_xml(name))
        subjects = getattr(item, schema.getRSSFieldId('subject'))
        if type(subjects) != type([]):
            subjects = [subjects]
        for subject in subjects:
            if subject:
                xml.write('<category term="%s" />' % quote_xml(subject))
        description = getattr(item, schema.getRSSFieldId('description'))
        if type(description) == type([]):
            description = description[0]
        if description:
            xml.write('<summary>%s</summary>' % quote_xml(description))
        xml.write('</entry>\n')

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'directory')
    def directory(self):
        """Return the contents of this query as an google sitemap file"""
        head = """
<html>
  <!-- An index for Google Scholar to find all the OAI Queries /-->
<head>
<meta name="ROBOTS" content="NOINDEX,FOLLOW">
<title>Directory Index</title>
</head>
<body>
        """
        foot = """
</body>
</html>
        """
        items = []
        for result in self.queryResults({}):
            items.append(
                '<a href="%s">%s</a><br />' % (
                self.absolute_url() + '/' + result.getObject().id,
                result.getObject().id))
        return head + '\n'.join(items) + foot
                
InitializeClass(Query)
    
manage_addQueryForm = PageTemplateFile(
    "www/queryAdd", globals(),
    __name__='manage_addQueryForm')

def manage_addQuery(self, id, title, service, REQUEST=None):
    """Add OAI query.
    """
    object = Query(id, title, service)
    self._setObject(id, object)
    object = getattr(self, id)
    object.set_title(title)
    return ''
